@@ -36,17 +36,12 @@
 #ifdef _WINDOWS
    #include <windows.h>
    #define PACKAGE "mysqltcl"
-   #define PACKAGE_VERSION "3.02"
+   #define PACKAGE_VERSION "3.04"
 #endif
 
 #include <tcl.h>
 #include <mysql.h>
 
-#if (MYSQL_VERSION_ID<40100)
-  #error You need Mysql version 4.1 or higher to compile mysqltcl
-#endif
-
-
 #include <errno.h>
 #include <string.h>
 #include <ctype.h>
@@ -97,6 +92,8 @@ static char *MysqlHandlePrefix = "mysql";
 #define MYSQL_STATUS_MSG  "message"
 #define MYSQL_STATUS_NULLV  "nullvalue"
 
+#define FUNCTION_NOT_AVAILABLE "function not available"
+
 /* C variable corresponding to mysqlstatus(nullvalue) */
 #define MYSQL_NULLV_INIT ""
 
@@ -309,12 +306,27 @@ static void mysql_reassemble(Tcl_Interp *interp,int objc,Tcl_Obj *CONST objv[])
    set_statusArr(interp,MYSQL_STATUS_CMD,Tcl_NewListObj(objc, objv));
 }
 
+/*
+ * free result from handle and consume left result of multresult statement 
+ */
+static void freeResult(MysqlTclHandle *handle)
+{
+	MYSQL_RES* result;
+	if (handle->result != NULL) {
+		mysql_free_result(handle->result);
+		handle->result = NULL ;
+	}
+#if (MYSQL_VERSION_ID >= 50000)
+	while (!mysql_next_result(handle->connection)) {
+		result = mysql_store_result(handle->connection);
+		if (result) {
+			mysql_free_result(result);
+		}
+	}
+#endif
+}
 
 /*
-
-
-
-
  *----------------------------------------------------------------------
  * mysql_prim_confl
  * Conflict handling after a primitive conflict.
@@ -473,10 +485,7 @@ static MysqlTclHandle *createHandleFrom(MysqltclState *statePtr,MysqlTclHandle *
 }
 static void closeHandle(MysqlTclHandle *handle)
 {
-  if (handle->result != NULL) {
-      mysql_free_result(handle->result);
-      handle->result = NULL;
-  }
+  freeResult(handle);
   if (handle->type==HT_CONNECTION) {
     mysql_close(handle->connection);
   }
@@ -549,7 +558,6 @@ static MysqlTclHandle *mysql_prologue(Tcl_Interp *interp,int objc,Tcl_Obj *CONST
 
 static Tcl_Obj *mysql_colinfo(Tcl_Interp *interp,int objc,Tcl_Obj *CONST objv[],MYSQL_FIELD* fld,Tcl_Obj * keyw)
 {
-  char buf[MYSQL_SMALL_SIZE];
   int idx ;
 
   static CONST char* MysqlColkey[] =
@@ -574,6 +582,7 @@ static Tcl_Obj *mysql_colinfo(Tcl_Interp *interp,int objc,Tcl_Obj *CONST objv[],
       switch (fld->type)
 	{
 
+
 	case FIELD_TYPE_DECIMAL:
 	  return Tcl_NewStringObj("decimal", -1);
 	case FIELD_TYPE_TINY:
@@ -610,7 +619,6 @@ static Tcl_Obj *mysql_colinfo(Tcl_Interp *interp,int objc,Tcl_Obj *CONST objv[],
 	  return Tcl_NewStringObj("set", -1);
 	case FIELD_TYPE_TINY_BLOB:
 	  return Tcl_NewStringObj("tiny blob", -1);
-
 	case FIELD_TYPE_MEDIUM_BLOB:
 	  return Tcl_NewStringObj("medium blob", -1);
 	case FIELD_TYPE_LONG_BLOB:
@@ -621,14 +629,16 @@ static Tcl_Obj *mysql_colinfo(Tcl_Interp *interp,int objc,Tcl_Obj *CONST objv[],
 	  return Tcl_NewStringObj("var string", -1);
 	case FIELD_TYPE_STRING:
 	  return Tcl_NewStringObj("string", -1);
+#if MYSQL_VERSION_ID >= 50000
+	case MYSQL_TYPE_NEWDECIMAL:
+	   return Tcl_NewStringObj("newdecimal", -1);
+	case MYSQL_TYPE_GEOMETRY:
+	   return Tcl_NewStringObj("geometry", -1);
+	case MYSQL_TYPE_BIT:
+	   return Tcl_NewStringObj("bit", -1);
+#endif
 	default:
-	  if (strlen(fld->name)<MYSQL_SMALL_SIZE-30) {
- 	    sprintf(buf, "column '%s' has unknown datatype", fld->name);
-	  } else {
- 	    sprintf(buf, "column has unknown datatype");
-	  }
-          mysql_prim_confl(interp,objc,objv,buf);
-	  return NULL ;
+	  return Tcl_NewStringObj("unknown", -1);
 	}
       break ;
     case MYSQL_COL_LENGTH_K:
@@ -713,7 +723,10 @@ static void Mysqltcl_Kill(ClientData clientData)
 static CONST char* MysqlConnectOpt[] =
     {
       "-host", "-user", "-password", "-db", "-port", "-socket","-encoding",
-      "-ssl", "-compress", "-noschema","-odbc","-multistatement","-multiresult",
+      "-ssl", "-compress", "-noschema","-odbc",
+#if (MYSQL_VERSION_ID >= 40107)
+      "-multistatement","-multiresult",
+#endif
       "-localfiles","-ignorespace","-foundrows","-interactive","-sslkey","-sslcert",
       "-sslca","-sslcapath","-sslciphers",NULL
     };
@@ -730,7 +743,9 @@ static int Mysqltcl_Connect(ClientData clientData, Tcl_Interp *interp, int objc,
   char *socket = NULL;
   char *encodingname = NULL;
 
+#if (MYSQL_VERSION_ID >= 40107)
   int isSSL = 0;
+#endif
   char *sslkey = NULL;
   char *sslcert = NULL;
   char *sslca = NULL;
@@ -745,7 +760,10 @@ static int Mysqltcl_Connect(ClientData clientData, Tcl_Interp *interp, int objc,
     MYSQL_CONNHOST_OPT, MYSQL_CONNUSER_OPT, MYSQL_CONNPASSWORD_OPT, 
     MYSQL_CONNDB_OPT, MYSQL_CONNPORT_OPT, MYSQL_CONNSOCKET_OPT, MYSQL_CONNENCODING_OPT,
     MYSQL_CONNSSL_OPT, MYSQL_CONNCOMPRESS_OPT, MYSQL_CONNNOSCHEMA_OPT, MYSQL_CONNODBC_OPT,
-    MYSQL_MULTISTATEMENT_OPT,MYSQL_MULTIRESULT_OPT,MYSQL_LOCALFILES_OPT,MYSQL_IGNORESPACE_OPT,
+#if (MYSQL_VERSION_ID >= 40107)
+    MYSQL_MULTISTATEMENT_OPT,MYSQL_MULTIRESULT_OPT,
+#endif
+    MYSQL_LOCALFILES_OPT,MYSQL_IGNORESPACE_OPT,
     MYSQL_FOUNDROWS_OPT,MYSQL_INTERACTIVE_OPT,MYSQL_SSLKEY_OPT,MYSQL_SSLCERT_OPT,
     MYSQL_SSLCA_OPT,MYSQL_SSLCAPATH_OPT,MYSQL_SSLCIPHERS_OPT
   };
@@ -786,8 +804,15 @@ static int Mysqltcl_Connect(ClientData clientData, Tcl_Interp *interp, int objc,
       encodingname = Tcl_GetStringFromObj(objv[++i],NULL);
       break;
     case MYSQL_CONNSSL_OPT:
+#if (MYSQL_VERSION_ID >= 40107)
       if (Tcl_GetBooleanFromObj(interp,objv[++i],&isSSL) != TCL_OK )
 	return TCL_ERROR;
+#else
+      if (Tcl_GetBooleanFromObj(interp,objv[++i],&booleanflag) != TCL_OK )
+	return TCL_ERROR;
+      if (booleanflag)
+        flags |= CLIENT_SSL;
+#endif
       break;
     case MYSQL_CONNCOMPRESS_OPT:
       if (Tcl_GetBooleanFromObj(interp,objv[++i],&booleanflag) != TCL_OK )
@@ -807,13 +832,12 @@ static int Mysqltcl_Connect(ClientData clientData, Tcl_Interp *interp, int objc,
       if (booleanflag)
 	flags |= CLIENT_ODBC;
       break;
+#if (MYSQL_VERSION_ID >= 40107)
     case MYSQL_MULTISTATEMENT_OPT:
       if (Tcl_GetBooleanFromObj(interp,objv[++i],&booleanflag) != TCL_OK )
 	return TCL_ERROR;
       if (booleanflag)
 	flags |= CLIENT_MULTI_STATEMENTS;
-
-
       break;
     case MYSQL_MULTIRESULT_OPT:
       if (Tcl_GetBooleanFromObj(interp,objv[++i],&booleanflag) != TCL_OK )
@@ -821,7 +845,7 @@ static int Mysqltcl_Connect(ClientData clientData, Tcl_Interp *interp, int objc,
       if (booleanflag)
 	flags |= CLIENT_MULTI_RESULTS;
       break;
-
+#endif
     case MYSQL_LOCALFILES_OPT:
       if (Tcl_GetBooleanFromObj(interp,objv[++i],&booleanflag) != TCL_OK )
 	return TCL_ERROR;
@@ -880,9 +904,11 @@ static int Mysqltcl_Connect(ClientData clientData, Tcl_Interp *interp, int objc,
 #if (MYSQL_VERSION_ID>=32350)
   mysql_options(handle->connection,MYSQL_READ_DEFAULT_GROUP,groupname);
 #endif
+#if (MYSQL_VERSION_ID >= 40107)
   if (isSSL) {
       mysql_ssl_set(handle->connection,sslkey,sslcert, sslca, sslcapath, sslcipher);
   }
+#endif
 
   if (!mysql_real_connect(handle->connection, hostname, user,
                                 password, db, port, socket, flags)) {
@@ -1026,12 +1052,9 @@ static int Mysqltcl_Sel(ClientData clientData, Tcl_Interp *interp, int objc, Tcl
 			    TCL_EXACT, &selOption) != TCL_OK)
       return TCL_ERROR;
   }
-       
+
   /* Flush any previous result. */
-  if (handle->result != NULL) {
-    mysql_free_result(handle->result) ;
-    handle->result = NULL ;
-  }
+  freeResult(handle);
 
   if (mysql_QueryTclObj(handle,objv[2])) {
     return mysql_server_confl(interp,objc,objv,handle->connection);
@@ -1134,10 +1157,7 @@ static int Mysqltcl_EndQuery(ClientData clientData, Tcl_Interp *interp, int objc
     }
     closeHandle(handle);
   } else {
-    if (handle->result != NULL) {
-    	mysql_free_result(handle->result) ;
-	handle->result = NULL ;
-    }
+      freeResult(handle);
   }
   return TCL_OK;
 }
@@ -1159,26 +1179,33 @@ static int Mysqltcl_EndQuery(ClientData clientData, Tcl_Interp *interp, int objc
 
 static int Mysqltcl_Exec(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
-  MysqlTclHandle *handle;
-  int affected;
+	MysqlTclHandle *handle;
+	int affected;
+	Tcl_Obj *resList;
+    if ((handle = mysql_prologue(interp, objc, objv, 3, 3, CL_CONN,"handle sql-statement")) == 0)
+    	return TCL_ERROR;
 
-  if ((handle = mysql_prologue(interp, objc, objv, 3, 3, CL_CONN,
-			    "handle sql-statement")) == 0)
-    return TCL_ERROR;
+  	/* Flush any previous result. */
+	freeResult(handle);
 
-  /* Flush any previous result. */
-  if (handle->result != NULL) {
-      mysql_free_result(handle->result) ;
-      handle->result = NULL ;
-  }
+	if (mysql_QueryTclObj(handle,objv[2]))
+    	return mysql_server_confl(interp,objc,objv,handle->connection);
 
-  if (mysql_QueryTclObj(handle,objv[2]))
-    return mysql_server_confl(interp,objc,objv,handle->connection);
+	if ((affected=mysql_affected_rows(handle->connection)) < 0) affected=0;
 
-
-  if ((affected=mysql_affected_rows(handle->connection)) < 0) affected=0;
-  Tcl_SetIntObj(Tcl_GetObjResult(interp),affected);  
-  return TCL_OK ;
+#if (MYSQL_VERSION_ID >= 50000)
+	if (!mysql_next_result(handle->connection)) {
+		resList = Tcl_GetObjResult(interp);
+		Tcl_ListObjAppendElement(interp, resList, Tcl_NewIntObj(affected));
+		do {
+			if ((affected=mysql_affected_rows(handle->connection)) < 0) affected=0;
+      		Tcl_ListObjAppendElement(interp, resList, Tcl_NewIntObj(affected));
+		} while (!mysql_next_result(handle->connection));
+		return TCL_OK;
+	}
+#endif
+	Tcl_SetIntObj(Tcl_GetObjResult(interp),affected);  
+	return TCL_OK ;
 }
 
 
@@ -1204,11 +1231,10 @@ static int Mysqltcl_Fetch(ClientData clientData, Tcl_Interp *interp, int objc, T
   Tcl_Obj *resList;
   unsigned long *lengths;
 
-  if ((handle = mysql_prologue(interp, objc, objv, 2, 2, CL_RES,
-			    "handle")) == 0)
+  if ((handle = mysql_prologue(interp, objc, objv, 2, 2, CL_RES,"handle")) == 0)
     return TCL_ERROR;
 
-  
+
   if (handle->res_count == 0)
     return TCL_OK ;
   else if ((row = mysql_fetch_row(handle->result)) == NULL) {
@@ -1414,10 +1440,7 @@ static int Mysqltcl_Receive(ClientData clientData, Tcl_Interp *interp, int objc,
   if (Tcl_ListObjLength(interp, objv[3], &listObjc) != TCL_OK)
         return TCL_ERROR;
   
-  if (handle->result != NULL) {
-    mysql_free_result(handle->result) ;
-    handle->result = NULL ;
-  }
+  freeResult(handle);
   
   if (mysql_QueryTclObj(handle,objv[2])) {
     return mysql_server_confl(interp,objc,objv,handle->connection);
@@ -1513,7 +1536,11 @@ static int Mysqltcl_Info(ClientData clientData, Tcl_Interp *interp, int objc, Tc
   static CONST char* MysqlDbOpt[] =
     {
       "dbname", "dbname?", "tables", "host", "host?", "databases",
-      "info","serverversion","serverversionid","sqlstate","state",NULL
+      "info","serverversion",
+#if (MYSQL_VERSION_ID >= 40107)
+      "serverversionid","sqlstate",
+#endif
+      "state",NULL
     };
   enum dboption {
     MYSQL_INFNAME_OPT, MYSQL_INFNAMEQ_OPT, MYSQL_INFTABLES_OPT,
@@ -1547,8 +1574,10 @@ static int Mysqltcl_Info(ClientData clientData, Tcl_Interp *interp, int objc, Tc
     break;
   case MYSQL_INFO:
   case MYSQL_INF_SERVERVERSION:
+#if (MYSQL_VERSION_ID >= 40107)
   case MYSQL_INFO_SERVERVERSION_ID:
   case MYSQL_INFO_SQLSTATE:
+#endif
   case MYSQL_INFO_STATE:
     break;
 
@@ -1605,12 +1634,14 @@ static int Mysqltcl_Info(ClientData clientData, Tcl_Interp *interp, int objc, Tc
   case MYSQL_INF_SERVERVERSION:
      Tcl_SetObjResult(interp, Tcl_NewStringObj(mysql_get_server_info(handle->connection),-1));
      break;
+#if (MYSQL_VERSION_ID >= 40107)
   case MYSQL_INFO_SERVERVERSION_ID:
 	 Tcl_SetObjResult(interp, Tcl_NewIntObj(mysql_get_server_version(handle->connection)));
 	 break;
   case MYSQL_INFO_SQLSTATE:
      Tcl_SetObjResult(interp, Tcl_NewStringObj(mysql_sqlstate(handle->connection),-1));
      break;
+#endif
   case MYSQL_INFO_STATE:
      Tcl_SetObjResult(interp, Tcl_NewStringObj(mysql_stat(handle->connection),-1));
      break;
@@ -1637,7 +1668,11 @@ static int Mysqltcl_BaseInfo(ClientData clientData, Tcl_Interp *interp, int objc
   char **option;
   static CONST char* MysqlInfoOpt[] =
     {
-      "connectparameters", "clientversion","clientversionid", NULL
+      "connectparameters", "clientversion",
+#if (MYSQL_VERSION_ID >= 40107)
+      "clientversionid",
+#endif
+      NULL
     };
   enum baseoption {
     MYSQL_BINFO_CONNECT, MYSQL_BINFO_CLIENTVERSION,MYSQL_BINFO_CLIENTVERSIONID
@@ -1667,9 +1702,11 @@ static int Mysqltcl_BaseInfo(ClientData clientData, Tcl_Interp *interp, int objc
   case MYSQL_BINFO_CLIENTVERSION:
     Tcl_SetObjResult(interp, Tcl_NewStringObj(mysql_get_client_info(),-1));
     break;
+#if (MYSQL_VERSION_ID >= 40107)
   case MYSQL_BINFO_CLIENTVERSIONID:
     Tcl_SetObjResult(interp, Tcl_NewIntObj(mysql_get_client_version()));
     break;
+#endif
   }
   return TCL_OK ;
 }
@@ -1865,8 +1902,10 @@ static int Mysqltcl_State(ClientData clientData, Tcl_Interp *interp, int objc, T
   int numeric=0 ;
   Tcl_Obj *res;
 
-  if (objc!=2 && objc!=3)
+  if (objc!=2 && objc!=3) {
       Tcl_WrongNumArgs(interp, 1, objv, "handle ?-numeric");
+      return TCL_ERROR;
+  }
 
   if (objc==3) {
     if (strcmp(Tcl_GetStringFromObj(objv[2],NULL), "-numeric"))
@@ -1984,6 +2023,10 @@ static int Mysqltcl_ChangeUser(ClientData clientData, Tcl_Interp *interp, int ob
 
 static int Mysqltcl_AutoCommit(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
+#if (MYSQL_VERSION_ID < 40107)
+  Tcl_AddErrorInfo(interp, FUNCTION_NOT_AVAILABLE);
+  return TCL_ERROR;
+#else
   MysqlTclHandle *handle;
   int isAutocommit = 0;
 
@@ -1996,6 +2039,7 @@ static int Mysqltcl_AutoCommit(ClientData clientData, Tcl_Interp *interp, int ob
   	mysql_server_confl(interp,objc,objv,handle->connection);
   }
   return TCL_OK;
+#endif
 }
 /*
  *----------------------------------------------------------------------
@@ -2007,6 +2051,10 @@ static int Mysqltcl_AutoCommit(ClientData clientData, Tcl_Interp *interp, int ob
 
 static int Mysqltcl_Commit(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
+#if (MYSQL_VERSION_ID < 40107)
+  Tcl_AddErrorInfo(interp, FUNCTION_NOT_AVAILABLE);
+  return TCL_ERROR;
+#else
   MysqlTclHandle *handle;
 
   if ((handle = mysql_prologue(interp, objc, objv, 2, 2, CL_CONN,
@@ -2016,6 +2064,7 @@ static int Mysqltcl_Commit(ClientData clientData, Tcl_Interp *interp, int objc,
   	mysql_server_confl(interp,objc,objv,handle->connection);
   }
   return TCL_OK;
+#endif
 }
 /*
  *----------------------------------------------------------------------
@@ -2027,6 +2076,10 @@ static int Mysqltcl_Commit(ClientData clientData, Tcl_Interp *interp, int objc,
 
 static int Mysqltcl_Rollback(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
+#if (MYSQL_VERSION_ID < 40107)
+  Tcl_AddErrorInfo(interp, FUNCTION_NOT_AVAILABLE);
+  return TCL_ERROR;
+#else
   MysqlTclHandle *handle;
 
   if ((handle = mysql_prologue(interp, objc, objv, 2, 2, CL_CONN,
@@ -2036,6 +2089,7 @@ static int Mysqltcl_Rollback(ClientData clientData, Tcl_Interp *interp, int objc
       mysql_server_confl(interp,objc,objv,handle->connection);
   }
   return TCL_OK;
+#endif
 }
 /*
  *----------------------------------------------------------------------
@@ -2047,6 +2101,10 @@ static int Mysqltcl_Rollback(ClientData clientData, Tcl_Interp *interp, int objc
 
 static int Mysqltcl_MoreResult(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
+#if (MYSQL_VERSION_ID < 40107)
+  Tcl_AddErrorInfo(interp, FUNCTION_NOT_AVAILABLE);
+  return TCL_ERROR;
+#else
   MysqlTclHandle *handle;
   int boolResult = 0;
 
@@ -2056,6 +2114,7 @@ static int Mysqltcl_MoreResult(ClientData clientData, Tcl_Interp *interp, int ob
   boolResult =  mysql_more_results(handle->connection);
   Tcl_SetObjResult(interp,Tcl_NewBooleanObj(boolResult));
   return TCL_OK;
+#endif
 }
 /*
 
@@ -2069,6 +2128,10 @@ static int Mysqltcl_MoreResult(ClientData clientData, Tcl_Interp *interp, int ob
 
 static int Mysqltcl_NextResult(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
+#if (MYSQL_VERSION_ID < 40107)
+  Tcl_AddErrorInfo(interp, FUNCTION_NOT_AVAILABLE);
+  return TCL_ERROR;
+#else
   MysqlTclHandle *handle;
   int result = 0;
 
@@ -2095,6 +2158,7 @@ static int Mysqltcl_NextResult(ClientData clientData, Tcl_Interp *interp, int ob
       Tcl_SetObjResult(interp, Tcl_NewIntObj(handle->res_count));
   }
   return TCL_OK;
+#endif
 }
 /*
  *----------------------------------------------------------------------
@@ -2106,6 +2170,10 @@ static int Mysqltcl_NextResult(ClientData clientData, Tcl_Interp *interp, int ob
 
 static int Mysqltcl_WarningCount(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
+#if (MYSQL_VERSION_ID < 40107)
+  Tcl_AddErrorInfo(interp, FUNCTION_NOT_AVAILABLE);
+  return TCL_ERROR;
+#else
   MysqlTclHandle *handle;
   int count = 0;
 
@@ -2115,6 +2183,7 @@ static int Mysqltcl_WarningCount(ClientData clientData, Tcl_Interp *interp, int
   count = mysql_warning_count(handle->connection);
   Tcl_SetObjResult(interp,Tcl_NewIntObj(count));
   return TCL_OK;
+#endif
 }
 /*
  *----------------------------------------------------------------------
@@ -2173,13 +2242,19 @@ static int Mysqltcl_NewNull(ClientData clientData, Tcl_Interp *interp, int objc,
  *    usage: mysql::setserveroption (-
  *
  */
+#if (MYSQL_VERSION_ID >= 40107)
 static CONST char* MysqlServerOpt[] =
     {
       "-multi_statment_on", "-multi_statment_off",NULL
     };
+#endif
  
 static int Mysqltcl_SetServerOption(ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[])
 {
+#if (MYSQL_VERSION_ID < 40107)
+  Tcl_AddErrorInfo(interp, FUNCTION_NOT_AVAILABLE);
+  return TCL_ERROR;
+#else
   MysqlTclHandle *handle;
   int idx;
   enum enum_mysql_set_option mysqlServerOption;
@@ -2210,6 +2285,7 @@ static int Mysqltcl_SetServerOption(ClientData clientData, Tcl_Interp *interp, i
   	mysql_server_confl(interp,objc,objv,handle->connection);
   }
   return TCL_OK;
+#endif
 }
 /*
  *----------------------------------------------------------------------
@@ -2225,7 +2301,11 @@ static int Mysqltcl_ShutDown(ClientData clientData, Tcl_Interp *interp, int objc
   if ((handle = mysql_prologue(interp, objc, objv, 2, 2, CL_CONN,
 			    "handle")) == 0)
     return TCL_ERROR;
+#if (MYSQL_VERSION_ID >= 40107)
   if (mysql_shutdown(handle->connection,SHUTDOWN_DEFAULT)!=0) {
+#else
+  if (mysql_shutdown(handle->connection)!=0) {
+#endif
   	mysql_server_confl(interp,objc,objv,handle->connection);
   }
   return TCL_OK;
